from enum import Enum


class Language(str, Enum):
    """ A plurk's language. Can be the following:
        'en': 'English'
        'tr_ch': '中文 (台灣)'
        'tr_hk': '中文 (香港)'
        'cn': '中文 (中国)'
        'ja': '日本語'
        'ca': 'Català'
        'el': 'Ελληνικά'
        'dk': 'Dansk'
        'de': 'Deutsch'
        'es': 'Español'
        'sv': 'Svenska'
        'nb': 'Norsk bokmål'
        'hi': 'Hindi'
        'ro': 'Română'
        'hr': 'Hrvatski'
        'fr': 'Français'
        'ru': 'Pусский'
        'it': 'Italiano '
        'he': 'עברית'
        'hu': 'Magyar'
        'ne': 'Nederlands'
        'th': 'ไทย'
        'ta_fp': 'Filipino'
        'in': 'Bahasa Indonesia'
        'pl': 'Polski'
        'ar': 'العربية'
        'fi': 'Finnish'
        'tr': 'Türkçe'
        'ga': 'Gaeilge'
        'sk': 'Slovenský'
        'uk': 'українська'
        'fa': 'فارسی
        'pt_BR': 'Português'
    """
    EN = 'en'
    TR_CH = 'tr_ch'
    TR_HK = 'tr_hk'
    CN = 'cn'
    JA = 'ja'
    CA = 'ca'
    EL = 'el'
    DK = 'dk'
    DE = 'de'
    ES = 'es'
    SV = 'sv'
    NB = 'nb'
    HI = 'hi'
    RO = 'ro'
    HR = 'hr'
    FR = 'fr'
    RU = 'ru'
    IT = 'it'
    HE = 'he'
    HU = 'hu'
    NE = 'ne'
    TH = 'th'
    TA_FP = 'ta_fp'
    IN = 'in'
    PL = 'pl'
    AR = 'ar'
    FI = 'fi'
    TR = 'tr'
    GA = 'ga'
    SK = 'sk'
    UK = 'uk'
    FA = 'fa'
    PT_BR = 'pt_BR'
